@@ -4,7 +4,7 @@ import sys
 
 API_ENDPOINT = '/api'
 
-__version__ = "1.2.1"
+__version__ = "1.2.6"
 __python_version__ = '.'.join([str(i) for i in sys.version_info[:3]])
 _DEBUG = False
 
@@ -41,7 +41,7 @@ class SecLevel:
 def sign_url(url, secret, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None):
     # Normalize parameters
     seclevel = seclevel or SecLevel.NONE
-    expires  = int(expires or time.time() + 7200)
+    expires = int(expires or time.time() + 7200)
 
     # Compute digest
     (url, unused, query) = url.partition('?')
@@ -81,8 +81,8 @@ def sign_url(url, secret, seclevel=None, asnum=None, ip=None, useragent=None, co
 
     public_secparams_encoded = ''
     if len(public_secparams) > 0:
-        public_secparams_encoded = base64.b64encode(zlib.compress('&'.join(public_secparams)))
-    rand   = ''.join(random.choice(string.ascii_lowercase + string.digits) for unused in range(8))
+        public_secparams_encoded = base64.urlsafe_b64encode(zlib.compress('&'.join(public_secparams)))
+    rand = ''.join(random.choice(string.ascii_lowercase + string.digits) for unused in range(8))
     digest = hashlib.md5('%d%s%d%s%s%s%s' % (seclevel, url, expires, rand, secret, secparams, public_secparams_encoded)).hexdigest()
 
     # Return signed URL
@@ -147,7 +147,31 @@ def sign(shared_secret, msg):
      >>> sign('sEcReT_KeY', 'hello world')
      '5f048ebaf6f06576b60716dc8f815d85'
     """
-    return hashlib.md5(msg.encode('utf-8') + shared_secret).hexdigest()
+    if isinstance(msg, unicode):
+        msg = msg.encode('utf8')
+    return hashlib.md5(msg + shared_secret).hexdigest()
+
+DRM_TOKEN_TIMEOUT = 600
+
+def gen_drm_token(user_id, media_id, api_key, rights=None, data=None, callback=None, expires=0, encode=True):
+    info = {
+        'user_id': user_id,
+        'media_id': media_id,
+        'expires': expires if expires and isinstance(expires, int) else int(time.time() + DRM_TOKEN_TIMEOUT),
+    }
+    if isinstance(rights, dict):
+        info['rights'] = rights
+    if isinstance(data, dict):
+        info['data'] = data
+    if isinstance(callback, (str, unicode)):
+        info['callback'] = callback
+
+    info['auth'] = sign(api_key, normalize(data))
+
+    payload = json.dumps(info)
+    if encode:
+        return base64.urlsafe_b64encode(payload)
+    return payload
 
 #################################################################################
 class RPCException(Exception):
@@ -288,7 +312,7 @@ class ClientObject(object):
 
 
             c = pycurl.Curl()
-            c.setopt(pycurl.URL, self._client._api_endpoint)
+            c.setopt(pycurl.URL, str(self._client._api_endpoint))
             c.setopt(pycurl.USERAGENT, 'cloudkey-py/%s (Python %s)' % (__version__, __python_version__))
             c.setopt(pycurl.HTTPHEADER, ["Content-Type: application/json", 'Expect:'])
 
@@ -305,7 +329,7 @@ class ClientObject(object):
                 data = json.dumps(request, cls=JSONEncoder)
             except (TypeError, ValueError), e:
                 raise SerializerError(str(e))
-                
+
             c.setopt(pycurl.POSTFIELDS, data)
 
             if self._client._proxy:
@@ -347,6 +371,9 @@ class FileObject(ClientObject):
             raise IOError("[Errno 2] No such file or directory: '%s'" % file)
         result = self.upload()
 
+        if isinstance(file, unicode):
+            file = file.encode('utf8')
+
         c = pycurl.Curl()
         c.setopt(pycurl.URL, str(result['url']))
         c.setopt(pycurl.USERAGENT, 'cloudkey-py/%s (Python %s)' % (__version__, __python_version__))
@@ -375,12 +402,27 @@ class FileObject(ClientObject):
 
 class MediaObject(ClientObject):
 
-    def get_embed_url(self, id, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, skin=None):
+    def _get_url(self, base_path, id, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, skin=None, drm_token=None, secure=False):
         if type(id) not in (str, unicode):
             raise InvalidParameter('id is not valid')
-        url = '%s/embed/%s/%s' % (self._client._base_url, self._client._user_id, id)
-        return sign_url(url, self._client._api_key, seclevel=seclevel, asnum=asnum, ip=ip, useragent=useragent, countries=countries, referers=referers, expires=expires) \
-            + ('skin=%s' % skin if skin else '')
+        base_url = self._client._secure_base_url if secure else self._client._base_url
+        url = '%s%s/%s/%s' % (base_url, base_path, self._client._user_id, id)
+        url = sign_url(url, self._client._api_key, seclevel=seclevel, asnum=asnum, ip=ip, useragent=useragent, countries=countries, referers=referers, expires=expires) \
+            + ('&skin=%s' % skin if skin else '')
+        if drm_token:
+            drm_token = gen_drm_token(self._client._user_id, id, self._client._api_key, \
+                                          expires=drm_token.get('expires'), \
+                                          rights=drm_token.get('rights'), \
+                                          data=drm_token.get('data'), \
+                                          callback=drm_token.get('callback'))
+            url += '&drm_token=%s' % drm_token
+        return url
+
+    def get_embed_url(self, id, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, skin=None, drm_token=None, secure=False):
+        return self._get_url('/embed', id, seclevel, asnum, ip, useragent, countries, referers, expires, skin, drm_token, secure)
+
+    def get_swf_url(self, id, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, skin=None, drm_token=None, secure=False):
+        return self._get_url('/player/swf', id, seclevel, asnum, ip, useragent, countries, referers, expires, skin, drm_token, secure)
 
     def get_qtref_url(self, id, seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None):
         if type(id) not in (str, unicode):
@@ -388,18 +430,30 @@ class MediaObject(ClientObject):
         url = '%s/stream/%s/%s.mov' % (self._client._base_url, self._client._user_id, id)
         return sign_url(url, self._client._api_key, seclevel=seclevel, asnum=asnum, ip=ip, useragent=useragent, countries=countries, referers=referers, expires=expires)
 
-    def get_stream_url(self, id, asset_name='mp4_h264_aac', seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, download=False, cdn_url='http://cdn.dmcloud.net'):
+    def get_stream_url(self, id, asset_name='mp4_h264_aac', seclevel=None, asnum=None, ip=None, useragent=None, countries=None, referers=None, expires=None, download=False, filename=None, version=None, protocol=None, cdn_url='http://cdn.dmcloud.net'):
         if type(id) not in (str, unicode):
             raise InvalidParameter('id is not valid')
+        if protocol not in (None, 'hls', 'rtmp', 'hps', 'http', 'ss', 'hds'):
+            raise InvalidParameter('%s is not a valid streaming protocol' % protocol)
+        version = '-%d' % version if version else ''
         if asset_name.startswith('jpeg_thumbnail_'):
             base_url = cdn_url.replace('cdn.', 'static.')
-            ts = '-%d' % int(expires) if expires else ''
-            return '%s/%s/%s/%s%s.jpeg' % (base_url, self._client._user_id, id, asset_name, ts)
-        extension = asset_name.split('_')[0]
-        if extension == 'f4f': extension = 'f4m'
-        url = '%s/route/%s/%s/%s.%s' % (cdn_url, self._client._user_id, id, asset_name, extension)
-        return sign_url(url, self._client._api_key, seclevel=seclevel, asnum=asnum, ip=ip, useragent=useragent, countries=countries, referers=referers, expires=expires) \
-            + ('&throttle=0&helper=0&cache=0' if download else '')
+            return '%s/%s/%s/%s%s.jpeg' % (base_url, self._client._user_id, id, asset_name, version)
+        extension = '.%s' % asset_name.split('_')[0]
+        if download or filename:
+            protocol = 'http'
+
+        if extension == '.abs':
+            extension = ''
+            if not protocol:
+                raise InvalidParameter('protocol is required for abs asset_name')
+
+        url = '%s/route%s/%s/%s/%s%s%s' % (cdn_url, '/%s' % protocol if protocol else '', self._client._user_id, id, asset_name, version, extension)
+
+        if filename:
+            url = '%s?%s' % (url, urllib.urlencode({'filename': filename.encode('utf-8', 'ignore')}))
+
+        return sign_url(url, self._client._api_key, seclevel=seclevel, asnum=asnum, ip=ip, useragent=useragent, countries=countries, referers=referers, expires=expires)
 
 
 class CloudKey(object):
@@ -409,6 +463,7 @@ class CloudKey(object):
         self._user_id = user_id if user_id else ''
         self._api_key = api_key if api_key else ''
         self._base_url = base_url
+        self._secure_base_url = base_url.replace('http:', 'https:')
         self._act_as_user = None
         self._api_endpoint =  base_url + API_ENDPOINT
         self._proxy = proxy
